@@ -1,12 +1,12 @@
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::sync::mpsc::channel;
-use std::time::{Duration, Instant};
+use std::time::Instant;
 
 use anyhow::Result;
 use clap::{Parser, ValueHint};
 use colored::Colorize;
 use log::{debug, error};
-use notify::{watcher, RecursiveMode, Watcher};
+use notify::{raw_watcher, RecursiveMode, Watcher};
 use rayon::prelude::*;
 use walkdir::DirEntry;
 
@@ -15,6 +15,7 @@ use ::rust_python_linter::linter::check_path;
 use ::rust_python_linter::logging::set_up_logging;
 use ::rust_python_linter::message::Message;
 use ::rust_python_linter::tell_user;
+use rust_python_linter::settings::Settings;
 
 #[derive(Debug, Parser)]
 #[clap(name = "rust-python-linter")]
@@ -30,7 +31,7 @@ struct Cli {
     no_cache: bool,
 }
 
-fn run_once(files: &[PathBuf], cache: bool) -> Result<Vec<Message>> {
+fn run_once(files: &[PathBuf], settings: &Settings, cache: bool) -> Result<Vec<Message>> {
     // Collect all the files to check.
     let start = Instant::now();
     let files: Vec<DirEntry> = files.iter().flat_map(iter_python_files).collect();
@@ -40,8 +41,14 @@ fn run_once(files: &[PathBuf], cache: bool) -> Result<Vec<Message>> {
     let start = Instant::now();
     let messages: Vec<Message> = files
         .par_iter()
+        .filter(|entry| {
+            !settings
+                .exclude
+                .iter()
+                .any(|exclusion| entry.path().starts_with(exclusion))
+        })
         .map(|entry| {
-            check_path(entry.path(), &cache.into()).unwrap_or_else(|e| {
+            check_path(entry.path(), settings, &cache.into()).unwrap_or_else(|e| {
                 error!("Failed to check {}: {e:?}", entry.path().to_string_lossy());
                 vec![]
             })
@@ -88,36 +95,43 @@ fn main() -> Result<()> {
 
     set_up_logging(cli.verbose)?;
 
+    // TODO(charlie): Can we avoid this cast?
+    let paths: Vec<&Path> = cli.files.iter().map(PathBuf::as_path).collect();
+    let settings = Settings::from_paths(paths)?;
+
     if cli.watch {
         // Perform an initial run instantly.
         clearscreen::clear()?;
         tell_user!("Starting linter in watch mode...\n");
 
-        let messages = run_once(&cli.files, !cli.no_cache)?;
+        let messages = run_once(&cli.files, &settings, !cli.no_cache)?;
         report_continuously(&messages)?;
 
         // Configure the file watcher.
         let (tx, rx) = channel();
-        let mut watcher = watcher(tx, Duration::from_secs(2))?;
+        let mut watcher = raw_watcher(tx)?;
         for file in &cli.files {
             watcher.watch(file, RecursiveMode::Recursive)?;
         }
 
         loop {
             match rx.recv() {
-                Ok(_) => {
-                    // Re-run on all change events.
-                    clearscreen::clear()?;
-                    tell_user!("File change detected...\n");
+                Ok(e) => {
+                    if let Some(path) = e.path {
+                        if path.to_string_lossy().ends_with(".py") {
+                            clearscreen::clear()?;
+                            tell_user!("File change detected...\n");
 
-                    let messages = run_once(&cli.files, !cli.no_cache)?;
-                    report_continuously(&messages)?;
+                            let messages = run_once(&cli.files, &settings, !cli.no_cache)?;
+                            report_continuously(&messages)?;
+                        }
+                    }
                 }
                 Err(e) => return Err(e.into()),
             }
         }
     } else {
-        let messages = run_once(&cli.files, !cli.no_cache)?;
+        let messages = run_once(&cli.files, &settings, !cli.no_cache)?;
         report_once(&messages)?;
     }
 
